/**
 * {聊天消息气泡组件}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/05/05 14:56:34
 */
import { css, html, Component, classMap } from '@bd/core'

import '../avatar/index.js'

class Bubble extends Component {
  static props = {
    type: 'text',
    content: '',
    voiceLength: 0,
    avatar: '',
    readOnly: true
  }

  static styles = [
    css`
      :host {
        display: block;
        width: 100%;
        font-size: 14px;
        color: var(--color-dark-1);
      }

      .container {
        display: flex;
        width: 100%;
        margin-top: 16px;

        .content {
          position: relative;
          margin-left: 16px;
          padding: 8px 16px;
          word-break: break-word;
          border-radius: 6px;
          background: #fff;

          a,
          img {
            -webkit-user-drag: none;
          }

          &::before {
            position: absolute;
            left: -6px;
            top: 10px;
            width: 12px;
            height: 12px;
            border-radius: 3px;
            background: #fff;
            transform: rotate(45deg);
            content: '';
          }
        }
      }

      :host([align='right']) {
        .container {
          flex-direction: row-reverse;

          .content {
            margin-left: unset;
            margin-right: 16px;
            background: #d9ecff;

            &::before {
              right: -6px;
              left: unset;
              background: #d9ecff;
            }
          }
        }
      }
    `,
    css`
      .img-thumb,
      ::slotted(img) {
        max-width: 128px;
        min-width: 32px;
        max-height: 160px;
        object-fit: contain;
        -webkit-user-drag: none;
      }

      .type-voice {
        display: flex;
        align-items: center;

        .wave {
          position: relative;
          display: flex;
          align-items: center;
          width: 52rpx;
          height: 44rpx;

          .i {
            position: absolute;
            width: 8rpx;
            height: 8rpx;
            border: 4rpx solid transparent;
            border-right-color: rgba(0, 0, 0, 0.2);
            border-radius: 50%;

            &:nth-child(1) {
              border: 0;
              background: rgba(0, 0, 0, 0.2);
            }
            &:nth-child(2) {
              left: -8rpx;
              width: 24rpx;
              height: 24rpx;
            }
            &:nth-child(3) {
              left: -12rpx;
              width: 36rpx;
              height: 36rpx;
            }
            &:nth-child(4) {
              left: -16rpx;
              width: 48rpx;
              height: 48rpx;
            }
          }
          &.on .i {
            border-right-color: rgba(0, 0, 0, 0);
            animation: play 1.2s infinite;

            &:nth-child(3) {
              animation-delay: 0.3s;
            }
            &:nth-child(4) {
              animation-delay: 0.6s;
            }
          }
        }
      }

      @keyframes play {
        from,
        to {
          border-right-color: rgba(0, 0, 0, 0);
        }
        50% {
          border-right-color: rgba(0, 0, 0, 0.2);
        }
      }
    `
  ]

  #playing = false

  #drawContent() {
    switch (this.type) {
      case 'text':
        return this.content

      case 'image':
        return html`<img class="img-thumb" src=${this.content} />`

      case 'voice':
        return html`<div class="type-voice">
          <div class=${classMap({ wave: true, on: this.#playing })}>
            <i class="i"></i>
            <i class="i"></i>
            <i class="i"></i>
            <i class="i"></i>
          </div>
          <span class="duration">${this.voiceLength}'</span>
        </div> `

      case 'media':
        return html`<audio src=""></audio>`
    }
  }

  render() {
    return html`
      <div class="container">
        <wc-avatar
          size="xl"
          round
          class="avatar"
          src=${this.avatar}
        ></wc-avatar>
        <div class="content">
          <slot>${this.#drawContent()}</slot>
        </div>
      </div>
    `
  }
}

Bubble.reg('chat-bubble')
